#ifndef __CAAM420_VECTDENSE__
#define __CAAM420_VECTDENSE__

#include "vectbase.hh"
extern "C" {
#include <gsl/gsl_vector.h>
}

/*
 * Author: WWS
 *
 * Purpose: templated GSL vector class 
 */

// note: outside of class declaration, must refer to class by 
// full name - includes template!
template<typename Scalar>
class VecTGSL: public VecTBase<Scalar> {
};

template<>
class VecTGSL<double>: public VecTBase<double> {

private:

  gsl_vector * gv;

  // disabled default and copy constructors
  VecTGSL();

public:

  /* main constructor - note use of default arg */
  VecTGSL(int n): VecTBase<double>() { gv = gsl_vector_alloc(n); }

  /* copy constructor */
  VecTGSL(VecTGSL<double> const & v) {
    gv=gsl_vector_alloc(v.getLength());
    gsl_vector_memcpy(gv,v.gv);
  }

  /* destructor - note that call to delete is safe. At moment contemplate
     no subclasses, so need not be virtual.
  */
  ~VecTGSL() { gsl_vector_free(gv); }

  int getLength() const { return (int)(gv->size); }

  double & operator()(int i) { 
    if (i >= 0 && i <= gv->size-1) return gv->data[i];
    else {
      CAAM420Exception e;
      e<<"Error: VecTGSL::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"out of range\n";
      throw e;
    }
  }

  double const & operator()(int i) const {
    if (i >= 0 && i <= gv->size-1) return gv->data[i];
    else {
      CAAM420Exception e;
      e<<"Error: VecTGSL::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"out of range\n";
      throw e;
    }
  }
  
  /* mandatory report function */
  virtual ostream & report(ostream & str) const {
    str<<"VecTGSL object: length="<<gv->size<<"\n";
    for (int i=0;i<gv->size;i++) {
      str<<"v["<<i<<"]="<<gv->data[i]<<"\n";
    }
    return str;
  }

  /************************ NEW THING ****************/
  /**
   * special implementation of axpy - try to do it with gsl vecs,
   * else default to base class implementation
   */

  void axpy(Scalar a, VecTBase<Scalar> const & x) {
    try {
      VecTGSL<Scalar> const & gx = dynamic_cast<VecTGSL<Scalar> const &>(x);
      if (this->gv.size != gx.gv.size) {
	CAAM420Exception e;
	e<<"Error: VecTBase::axpy\n";
	e<<"incompatible vectors:\n";
	e<<"length of this = "<<this->getLength()<<"\n";
	e<<"length of that = "<<x.getLength()<<"\n";
	throw e;
      }
      for (int i=0;i<this->gv.size;i++) this->gv.data[i]+=a*(gx.gv.data[i]);
    }
    // standard bad_cast exception
    catch (bad_cast) {
      VecTBase::axpy(a,x);
    }
  }

};

#endif


