#include <stdlib.h>
#include <stdio.h>
#include <math.h>

int main(int argc, char ** argv) {

  if (argc !=4) {
    printf("Histogram generator for sample mean of C std library function\n");
    printf("random(), scaled to produce pseudorandom numbers in the range\n");
    printf("[-1,1]. Computes mean over a sample of prescribed size, and\n");
    printf("samples the resulting distribution of the mean over a prescribed\n");
    printf("number of trials. Means scaled by square root of number of trials,\n");
    printf("so that limit of infinitely many trials is Gaussian (CLT). \n");
    printf("Counts means in each of a prescribed number of equal-sized bins\n");
    printf("(subintervals of [-1,1]), and prints result to screen.\n\n");
    printf("usage: [prog name] [sample size] [number of trials] [number of histogram bins]\n");
    exit(1);
  }

  /*
   * Declarations 
   */
  int nsamp;    // size of each sample
  int ntrial;   // number of samples to generate
  int nbin;     // number of bins in histogram
  int itrial;   // sample counter
  int isamp;    // in-sample counter
  int ibin;     // bin counter
  float * mean; // array of ntrial means
  float * hist; // array of nbin counts
  float lbin;   // workspace for left endpoint of bin
  float dbin;   // bin width
  float rmax;   // max output of random (2^31-1), as float
  float scale;  // scale factor to approximate central tendency

  /*
   * Statements
   */

  // set random seed by process number
  srandom(getpid());

  // compute max output of random = 2^31-1 (clunky!)
  rmax=1;
  for (ibin=0;ibin<31;ibin++) rmax*=2.0f;
  rmax-=1.0f;

  // convert intputs to integers
  nsamp=atoi(argv[1]);
  ntrial=atoi(argv[2]);
  nbin=atoi(argv[3]);

  // set bin left endpoint, width
  lbin=-1.0f;
  dbin=2.0f/((float)nbin);

  // allocate workspace for histogram, sample mean vectors
  hist = (float *)malloc(nbin*sizeof(float));
  if (!hist) {
    printf("Error: failed to allocate memory for histogram\n");
    exit(1);
  }
  mean = (float *)malloc(ntrial*sizeof(float));
  if (!mean) {
    printf("Error: failed to allocate memory for means\n");
    exit(1);
  }

  // initialize these
  for (ibin=0;ibin<nbin;ibin++) hist[ibin]=0.0f;
  for (itrial=0;itrial<ntrial;itrial++) mean[itrial]=0.0f;

  // generate sample of ntrial sums of nsamp random nos.
  for (itrial=0;itrial<ntrial;itrial++)
    for (isamp=0;isamp<nsamp;isamp++)
      mean[itrial]+=2.0f*((float)(random())/rmax)-1.0f;
    
  // scale by sqrt(nsamp) per central limit theorem - combines
  // division by nsamp to convert sums to means, and multiplication
  // by sqrt(nsamp) 
  scale=1.0f/sqrt((float)nsamp);
  for (itrial=0;itrial<ntrial;itrial++) mean[itrial]*=scale;

  // bin means to compute histogram
  for (itrial=0;itrial<ntrial;itrial++) {
    lbin=-1.0f;
    for (ibin=0;ibin<nbin;ibin++) {
      if ((lbin<=mean[itrial]) &&
	  (lbin+dbin>mean[itrial])) hist[ibin]+=1.0f/((float)ntrial);
      lbin+=dbin;
    }
  }

  // output to terminal - first element of each pair is center of bin
  lbin=-1.0f+0.5*dbin;
  for (ibin=0;ibin<nbin;ibin++) printf("%10.2e %10.2e\n",lbin+ibin*dbin,hist[ibin]);

  // clean up
  free(hist);
  free(mean);

  return 0;
}

