#include <stdlib.h>
#include <stdio.h>
#include <math.h>

int main(int argc, char ** argv) {

  if ((argc !=4)&&(argc!=5)) {
    printf("Histogram generator for sample mean of C std library function\n");
    printf("random(), scaled to produce pseudorandom numbers in the range\n");
    printf("[-1,1]. Computes mean over a sample of prescribed size, and\n");
    printf("samples the resulting distribution of the mean over a prescribed\n");
    printf("number of trials. Means scaled by square root of number of trials,\n");
    printf("so that limit of infinitely many trials is Gaussian (CLT). \n");
    printf("Counts means in each of a prescribed number of equal-sized bins\n");
    printf("(subintervals of [-1,1]). Executes this procedure a prescribed number\n");
    printf("of times [default=1], and prints the resulting histogram realizations\n");
    printf("to the file \"hist.dat\" in the form of x-y pairs, x being bin center and y being \n");
    printf("relative bin count, one pair to a line, with two blank lines\n");
    printf("between successive realizations.\n\n");
    printf("usage: [prog name] [sample size] [number of trials] [number of histogram bins] [optional number of realizations - default=1]\n");
    exit(1);
  }

  /*
   * Declarations 
   */
  int nsamp;    // size of each sample
  int ntrial;   // number of samples to generate
  int nbin;     // number of bins in histogram
  int nreal;    // number of realizations
  int itrial;   // sample counter
  int isamp;    // in-sample counter
  int ibin;     // bin counter
  int ireal;    // realization counter
  float * mean; // array of ntrial means
  float * hist; // array of nbin counts
  float lbin;   // workspace for left endpoint of bin
  float dbin;   // bin width
  float rmax;   // max output of random (2^31-1), as float
  float scale;  // scale factor to approximate central tendency
  FILE * fp;    // output stream

  /*
   * Statements
   */

  // set random seed by process number
  srandom(getpid());

  // compute max output of random = 2^31-1 (clunky!)
  rmax=1;
  for (ibin=0;ibin<31;ibin++) rmax*=2.0f;
  rmax-=1.0f;

  // convert intputs to integers
  nsamp=atoi(argv[1]);
  ntrial=atoi(argv[2]);
  nbin=atoi(argv[3]);

  // default 
  nreal=1; 
  if (argc==5) nreal=atoi(argv[4]);

  // sanity tests
  if (nsamp<0) {
    printf("Error: sample N for each mean must be > 0, was %d\n",nsamp);
    exit(1);
  }
  if (ntrial<0) {
    printf("Error: number of means must be > 0, was %d\n",ntrial);
    exit(1);
  }
  if (nbin<0) {
    printf("Error: number of bins must be > 0, was %d\n",nbin);
    exit(1);
  }
  if (nreal<0) {
    printf("Error: number histogram realizations must be > 0, was %d\n",nreal);
    exit(1);
  }

  // set bin left endpoint, width
  lbin=-1.0f;
  dbin=2.0f/((float)nbin);

  // allocate workspace for histogram, sample mean vectors
  hist = (float *)malloc(nbin*sizeof(float));
  if (!hist) {
    printf("Error: failed to allocate memory for histogram\n");
    exit(1);
  }
  mean = (float *)malloc(ntrial*sizeof(float));
  if (!mean) {
    printf("Error: failed to allocate memory for means\n");
    exit(1);
  }

  // open file
  if (!(fp=fopen("hist.dat","w"))) {
    printf("Error: failed to open output file \"hist.dat\"\n");
    exit(1);
  }

  /*******************
   * realization loop
   *******************/

  for (ireal=0;ireal<nreal;ireal++) {

    // initialize means, histogram
    for (ibin=0;ibin<nbin;ibin++) hist[ibin]=0.0f;
    for (itrial=0;itrial<ntrial;itrial++) mean[itrial]=0.0f;

    // generate sample of ntrial sums of nsamp random nos.
    for (itrial=0;itrial<ntrial;itrial++)
      for (isamp=0;isamp<nsamp;isamp++)
	mean[itrial]+=2.0f*((float)(random())/rmax)-1.0f;
    
    // scale by sqrt(nsamp) per central limit theorem - combines
    // division by nsamp to convert sums to means, and multiplication
    // by sqrt(nsamp) 
    scale=1.0f/sqrt((float)nsamp);
    for (itrial=0;itrial<ntrial;itrial++) mean[itrial]*=scale;

    // bin means to compute histogram
    for (itrial=0;itrial<ntrial;itrial++) {
      lbin=-1.0f;
      for (ibin=0;ibin<nbin;ibin++) {
	if ((lbin<=mean[itrial]) &&
	    (lbin+dbin>mean[itrial])) hist[ibin]+=1.0f/((float)ntrial);
	lbin+=dbin;
      }
    }

    // output to terminal - first element of each pair is center of bin
    lbin=-1.0f+0.5*dbin;
    for (ibin=0;ibin<nbin;ibin++) fprintf(fp,"%10.2e %10.2e\n",lbin+ibin*dbin,hist[ibin]);

    // tack on two blank lines for plotting purposes
    fprintf(fp,"\n\n");

  }

  // clean up
  free(hist);
  free(mean);

  fclose(fp);

  return 0;
}

