/*
 * Author: WWS
 *
 * Purpose: illustrate the use of structure for a simple
 * scientific data type
 */

#include "cstd.h"

/*
 * error flags - have same meaning throughout
 * note that if more than one file were involved,
 * these should be located in a header which can 
 * be included in all files
 */

#define E_ALLOC 1
#define E_BOUNDS 2

/*
 * an array that knows its length - arguably, a vector
 */
struct vec {
  int len;
  float * data;
};

/*
 * sets a vector to its initial state - should be called
 * automatically when a vector is created, but C provides 
 * no mechanism to do this, so you have to do it by hand
 *
 * Note the use of the "->" syntax - this is equivalent to
 * dereferencing followed by member access - that is, we
 * could as well have written
 *
 * (*v).len=0;
 * (*v).data=NULL;
 *
 * The parens are necessary to force the compiler to dereference THEN
 * find member - the structure member operator "." has higher
 * precedence than almost anything else, so "*v.len" first tries to
 * find the member len of the struct v - but v is not a struct, but a
 * pointer to one.
 *
 */
void vec_construct(struct vec * v) {
  v->len=0;
  v->data=NULL;
}

/*
 * I wrote this initialization function so that the "ideal" initial
 * status is verified - making memory leaks more unlikely. Otherwise,
 * you could call this function repeatedly without freeing the data
 * member. Each time, you would lose control of the previously 
 * allocated block of memory (which data pointed to, before being
 * overwritten!).
 */
int vec_init(struct vec * v, int n) {
  if ((v->len !=0) || (v->data !=NULL)) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  vector not in initial state - cannot create\n");
    return E_ALLOC;
  }
  if (n<1) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  length requested is %d - must be > 0\n",n);
    return E_BOUNDS;
  }
  v->len=n;
  if (!(v->data=(float *)malloc(n*sizeof(float)))) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  failed to allocated %d floats\n",n);
    return E_ALLOC;
  }
}

/* after destruction, you can re-create another vector in the same
 * variable. Since the data memory has been freed, it will likely be
 * re-used (but not necessarily - some other part of the process might
 * use it.)
 */
void vec_destroy(struct vec * v) {
  if (v->data) free(v->data);
  vec_init(v);
}

/*
 * you can access the components of v by v.data[i] - the point
 * of this function is the bounds check. That is,
 *
 * x=vec_get(v,i);
 * 
 * gives the same result as 
 *
 * x=v.data[i];
 *
 * if i is in range, but gives you an error message instead of a 
 * seg fault otherwise.
 */
float vec_get(struct vec v, int i) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_index, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    /*
     * what can we do here? in C, the only way to handle the error
     * appears to be by stopping the program, since we can't pass
     * an error flag back as the return value.
     */
    exit(1);
  }
  return v.data[i];
}

/* 
 * same story - why is it sufficient to pass a vec rather than 
 * a vec *?
 */
float vec_set(struct vec v, int i, float x) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_index, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    exit(1);
  }
  v.data[i]=x;
}


/* a simple display function */
void vec_print(struct vec v) {
  int i;
  printf("Vector structure: dimension=%d\n",v.len);
  for (i=0;i<v.len;i++) 
    printf("i=%d v[i]=%e\n",i,vec_get(v,i));
}

/* exercise this thing */
int main(int argc, char ** argv) {

  int i;               // counter
  int n=10;            // vector size
  struct vec v;        // a vec to play with

  vec_construct(&v);   // set to initial state
  vec_init(&v,n);      // make a vec of length 10
  for (i=0;i<n;i++)    // fill it with floats
    vec_set(v,i,(float)i);
  vec_print(v);        // print it out

  // some busts:

  // 1. out-of-bounds access, two ways
  //vec_set(v,11,11.0f);
  //v.data[110000000]=11.0f;

  // 2. attempt at memory leak
  // no memory leak
  //vec_destroy(&v);
  vec_create(&v,22);

  exit(0);
}
