/* 
 * Author: WWS 
 * Purpose: illustrate conventions for accessing Fortran subroutines
 * from C.
*/

#include <stdio.h>

/* declaration of the Fortran matvec subroutine - note that all
   arguments are pointers, and the name is "mangled" by appending
   an underscore - this is what the compiler produces in its output
   object code, and what the linker expects (see nm output).
*/
void matmult_(int*, int*, float*, float*, float*);

int main() {

  const int ncols = 5;    /* number of columns */
  const int nrows = 5;    /* number of rows    */
  int nr,nc;              /* nonconst versions */
  int i,j;                /* loop counters     */
  float a[nrows*ncols];   /* col-major storage */
  float b[nrows];         /* output vector     */
  float x[ncols];         /* input vector      */

  /* assign matrix entries - loop over rows, cols 
   * with loop scopes explicitly indicated by braces 
   * NOTE: this matrix is the famous HILBERT MATRIX,
   * 5x5 instance
   */
  for (i=0;i<nrows;i++) 
    for (j=0;j<ncols;j++) 
      a[i+j*nrows]=1.0f/((float)(i+j+1));

  /* print matrix */

  for (i=0;i<nrows;i++) {
    for (j=0;j<ncols;j++) printf("a(%d,%d)=%12.4e ",i,j,a[i+j*nrows]);
    printf("\n");
  }
  printf("\n");

  /* define x (input vector), print */
  for (j=0;j<ncols;j++) {
    x[j]=j+1;
    printf("x(%d)=%12.4e\n",j,x[j]);
  }
  printf("\n");

  /* multiply x by a, store in b, print result 
   * Three things to notice: 
   * (1) everything argument is passes as a pointer - a, x, and b
   * are already pointers, so you don't have to do anything, but nrows
   * and ncols are not
   * (2) the matrix is simply passed by pointer - Fortran expects 
   * column-major storage
   * (3) the subroutine name must have an underscore appended to be
   * correctly interpreted by the C compiler
   *
   * Note that passing the const ints generates a compiler warning - 
   * to get rid of it, use non-const variables with same values - this
   * happens because Fortran does not understand const args.
   */
  //  matmult_(&nrows,&ncols,a,x,b);
  //  /*
  nr=nrows;
  nc=ncols;
  matmult_(&nr,&nc,a,x,b);
  //  */

  /* print out the result */
  for (i=0;i<nrows;i++) {
    printf("b(%d)=%12.4e\n",i,b[i]);
  }

  return 0;
}
