/*
 * Author: WWS
 *
 * Purpose: illustrate some of the "better C" features of C++
 *
 */

/* Like cstd.h. 
 * Common suffixes for C++ header files: .H, .hh, .hpp, and even .h
 */
#include "cppstd.hh"

/*
 * Native pass by reference - compare with pass by value 
 * syntax: reference to x
 *   double & y = x;
 * where x already declared double. A reference is another name for 
 * for an object, which implies that the object must exist.
 *
 * Note that a pointer may not point to anything, but a reference must
 * refer to something. Thus references are safer than pointers.
 *
 * References are most useful to pass large object args, even when 
 * they will not be changed - in that case, they may (should) be declared
 * const, i.e.
 *   double const & y = x;
 */

// the extern "C" declaration is necessary because of "name mangling" -
// the C++ compiler assigns different (more complex) names than the C 
// compiler does, so if you're going to compile a component with C then
// link with C++ you need to tell C++ about it.
extern "C" int sqrtfun1( double *);

// extern "C++" is redundant
// reference note the reference syntax in the argument, 
// bool return type
bool sqrtfun2( double &);

int main(int argc, char ** argv) {

  double x = 2.0;
  double y = x;

  if (sqrtfun1(&x)) {
    printf("no sqrts of neg numbers\n");
  }
  else {
    printf("sqrt(%14.6e) = %14.6e\n",y,x);
  }

  // note that ref in arg list automaticlly extracts reference
  // to x and passes that - no need to declare reference then pass

  // start over
  x=y;
  if (sqrtfun2(x)) {
    printf("still no sqrts of neg numbers\n");
  }
  else {
    printf("sqrt(%14.6e) still = %14.6e\n",y,x);
  }

}
