c     This sample fortran program exhibits a number of similarities
c     and contrasts between fortran and c. Some ways in which fortran 
c     differs from c:
c
c     1. The column in which a word begins has significance in f77,
c     a heritage from the days when fortran was punched on Hollerith 
c     cards. [This is not true of f90, which is more free-form, like c.]
c
c     2. comments begin with "c" or "C" or "*" or "!" in the first 
c     column. A "!" anywhere in a line makes the rest of the line a 
c     comment [this is an f90 feature which seems to have been adopted
c     by most modern f77 compilers - it is an extension of the f77
c     standard].
c
c     3. in general lines should be no more than 72 characters long; some
c     f77 compilers let you transgress this rule with impunity, but some
c     don't and it is part of the standard [again, this is card reader 
c     legacy stuff and does not apply to f90]. Lines don't just continue,
c     i.e. f77 does NOT regard a newline as white space. To continue a 
c     line, you put any character in the sixth column, which is reserved for 
c     this purpose.
c
c     4. programs should begin with "program <name>" (think like main())
c
c     5. like c, all declarations must appear before all executable statements
c     in a program or subprogram block.
c
c     6. unlike c, f77 compilers are case-insensitive: foo, FOO, Foo,
c     fOo, and fOO all signify the SAME variable name! Old-fashioned fortran
c     was written in all caps, since that's what the punch cards encoded.
c     Starting with the 1966 standard, lower case letters were admitted,
c     and you should make case choices for readability.
c
c     7. Two big differences: statements do not end with semi-colons - they
c     just end, at the end of a line (or a continued line, via the sixth
c     column trick, mentioned above and illustrated below). [f90 permits 
c     you to end a statement with a semi-colon.] Blocks begin with "program",
c     "subroutine", or "function" statements, and end with "end" - no curly
c     braces.
c     
c     Other similarities and differences are noted below.

      program ex1

c     "hello world", and some other things

c     fortran has a dangerous feature, which is a holdover from 
c     the early days when folks didn't know how to program: certain
c     letters signify default types. Any word beginning with i-n 
c     is assumed to be the name of a variable of integer type. Any
c     word beginning with any other letter is assume to be the name
c     of a variable of type real, i.e. single precision float. NEVER,
c     NEVER TAKE ADVANTAGE OF THIS LANGUAGE FEATURE!!! It leads to 
c     difficult-to-kill bugs. Instead, always put this line at the 
c     top of your declaration list in ever block (program, subroutine,
c     function) - it is a common language extension which disables
c     the implicit typing feature, forcing you to declare all variables
c     as in c.

      implicit none

c     declarations, parameter statements

      integer n                ! c int
      integer ierr

c     single and double precision floating point declarations look like
c     this:
c      real x,y,z               ! c float
c      double precision xx      ! c double

      integer dim
      parameter (dim=10)       ! parameter like const - can be used as dim
      real a(dim)              ! array declaration

c     executable statements

c     Here are two ways to print to the screen:
c     (1) using the "write(unit,format,...) (list)" statement
c     if format = *, then write uses "list-directed" default formats

      write(6,*)'hello, world' ! unit 6 = standard out

c     (2) using "print format, (list)"
      print*, 'hello again'

c     note that the stuff to be printed is NOT an argument of
c     write or print (unlike printf), but tags on as a list,
c     and that strings are enclosed in single, not double, quotes.

c     standard loop syntax

      do n=1,10                ! like for (i=1;i<=10;i++)
         a(n) = n**2
      end do

      write(6,*)' '            ! one way to get a blank line

c     list directed write:

      do n=1,10
         write(6,*)
     &        n,a(n)           ! line continuation
      end do

      write(6,*)' '

c     formatted write:

      do n=1,10
         write(6,1000) n,a(n)
      end do
      
c     formatted writes are controlled by format statements, numbered in 
c     columns 2-5. These can appear anywhere in a block, though the
c     traditional location is at the end; they do not have to appear 
c     after the "write" statement that references them.
c     some standard formats: i = integer, f = fixed point, e = scientific
c     fn.m: n=field width, m=number of decimal digits
c     x = space
c     number prefix: repeat (eg. 4x = four spaces)

 1000 format(i6,4x,e12.4)
      
c     writing to a file requires an open statement, with function 
c     analogous to fopen in c. Fortran open takes a number of 
c     arguments in hash table form, i.e. key=value. Some of the
c     most useful are indicated here:
c
c     open(unit=unitnumber, file=filename, status=neworold, iostat=ierr)
c
c     the unitnumber acts like the file pointer FILE * fp, in 
c     that the file is subsequently referenced by it. The filename
c     is a string, just as in c (though if quoted literally, 
c     enclosed in single rather than double quotes). The status
c     word is 'new', 'old', or 'unknown'; it has function similar to
c     that of the arguments "r" and "w" of fopen. With status=old, you can
c     read or write - but the file must exist, otherwise an
c     error condition occurs. If status=new, the file is created
c     but must not already exist (else an error condition occurs), 
c     and writing begins at the beginning 
c     of the file. If status='unknown', then the code does whatever makes
c     sense - opens it if it already exists, creates it if it doesn't.
c     In many situations that's the sensible option (like 'w+' for fopen).
c
c     An error condition is signified by a nonzero
c     value of the iostat variable.

c     VERY IMPORTANT: units 5, 6, and 7 are predefined and have special 
c     meaning:
c
c     fortran       c        meaning
c     unit=5        stdin    input from terminal
c     unit=6        stdout   buffered output to screen (normal messages)
c     unit=7        stderr   unbuffered output (error messages)
c
c     a difference is that stderr in c goes to the screen, whereas
c     unit=7 in fortran goes to a file (fort.7, unless open(unit=7,...)
c     has been executed).
c
c     DO NOT OPEN ANYTHING ON UNITS 5 and 6!!!!!

      open(unit=1,file='plot.dat',status='unknown',iostat=ierr)

c     standard syntax for conditional branches - comparisons are
c     done differently than in c. The equivalents are:
c
c     c             fortran
c     ==            .eq.
c     !=            .ne.
c     <             .lt.
c     <=            .le.
c     >             .gt.
c     >=            .ge.
c
c     fortran has a BOOLEAN type - integers and true/false are not
c     confounded. The two values of a boolean variable are .TRUE.
c     and .FALSE.

      if (ierr.ne.0) then
         write(6,*)'Error: ex1 from open, file = plot.dat, ierr = ',ierr
      else

         do n=1,dim
            write(1,1000) n,a(n)
         end do
         close(1)             ! closes the file open on unit 1

         open(unit=1,file='plot.input',status='unknown',iostat=ierr)

         if (ierr.ne.0) then
            write(6,*)'Error: ex1 from open, file = plot.input, ierr = ',ierr
         else 

            write(1,*)'set xlabel "numbers"'
            write(1,*)'set ylabel "other numbers"'
            write(1,*)'set title "A Gnuplot Plot"'
            write(1,*)'plot "plot.dat" with lines'
            write(1,*)'pause -1'
            close(1)

c     system calls are also available in fortran - note the 
c     syntax for subroutine calls, with "call" preceding the
c     subroutine name.

            call system('gnuplot plot.input')

         end if

      end if

c     just as in c you could have this program clean up after itself

c      call system('/bin/rm -f plot.input plot.dat')

      stop
      end

