#include <stdio.h>
#include <stdlib.h>

// WARNING: NO ERROR CHECKING


// forward declaration of smart mydouble class
class mydouble;

// example skinny base class for a generic vector 
class vector {
protected:
  // single definining characteristic of a variable: length
  int dim;
public:

  // default constructor (zero length)
  vector(){
    dim = 0;
  }

  // construct with length (no storage)
  vector(int in_dim){
    dim = in_dim;
  }

  // pure virtual unary [] operator 
  virtual mydouble operator[] (int id) = 0;

  // pure virtual unary () operator 
  virtual double operator() (int id) const = 0;

  // pure virtual set function
  virtual void set(int id, double val) = 0;

  // implemented length function
  int length(){
    return dim;
  }

  virtual void report() = 0;

};


// "smart" mydouble class object to provide active entry 
// access to entry in vector
class mydouble {

private:
  // will use generic pointer
  // [ uses the pointer compatibility of classes derived from vector ]
  vector *spv;
  int index;
public:

  // default constructor
  mydouble(){
    spv = NULL;
    index = 0;
  }
  
  // constructor 
  mydouble(vector *in_spv, int in_index){
    spv = in_spv;
    index = in_index;
  }

  // type conversion opeartor: mydouble -> double
  operator double () const{
    return (*spv)(index);
  }  

  // assignment operator (mydouble a = 1.2)
  mydouble &operator = (const double val){
    
    spv->set(index, val);
    
    return *this;
  }

};



// dense vector class derived from vector base class
class dvector : public vector {
private:
  double *data;
public:
  
  // default constructor
  dvector() : vector() {
    data = NULL;
  }
    
  // constructor to set length and data
  dvector(int in_dim) : vector(in_dim) {
    data = new double[dim];
  }
  
  void set(int index, double val){
    data[index] = val;
  }

  // implementation of unary [] operator 
  mydouble operator[] (int id){
    mydouble pt(this, id);
    return pt;
  }
  
  // implementation of unary () operator
  double operator() (int id) const{
    return data[id];
  }

  // output entries
  void report(){
    for(int i=0;i<dim;++i)
      printf("d[%d] = %g\n", i, data[i]);
  } 

};

// sparse vector class derived from vector base class
class spvector : vector {
private:
  int nnz;
  int *indices;
  double *data;
public:

  // constructor for empty sparse vector
  spvector(int in_dim) : vector(in_dim){
    nnz = 0;
    indices = NULL;
    data = NULL;
  }

  // implementation of set function 
  // checks for existing entry and
  // extends storage for new entries
  void set(int index, double val){
    int i;

    for(i=0;i<nnz;++i){
      if(indices[i] == index){
	data[i] = val;
	return;
      }
    }
    
    // check to see if non-zero
    if(!val) return;

    // extend list by one
    // oops - realloc is not appropriate here 
    ++nnz;
    double *tmpdata = new double[nnz];
    for(i=0;i<nnz-1;++i) tmpdata[i] = data[i];
    tmpdata[i] = val;

    delete data;
    data = tmpdata;

    // oops need to realloc indices
    int *tmpindices = new int[nnz];
    for(i=0;i<nnz-1;++i) tmpindices[i] = indices[i];
    tmpindices[i] = index;
    
    delete indices;
    indices = tmpindices;

  }

  // implementation of operator () const
  double operator()(int index) const {
    
    for(int i=0;i<nnz;++i){
      if(indices[i] == index){
	return data[i];
      }
    }

    return 0;
  }

  // return smart mydouble for access to sparse entry
  mydouble operator[] (int index) {

    mydouble pt(this, index);
    return pt;
    
  }

  // output stored entries
  void report(){
    for(int i=0;i<nnz;++i)
      printf("nnz: %d, index: %d, val: %g\n", i, indices[i], data[i]);
  }

};


// some demos
main(){
  // create an empty sparse vector of length 10
  spvector sp_v(10);

  // set non-zero entries
  for(int i=0;i<10;i+=2)
    sp_v[i] = -2.4*i ;

  // read value
  double a = sp_v[3];
  
  // print out copied value and example read value
  printf("a = %f \n v(4) = %f\n", a, (double)sp_v[4]);
  
  // report non-zero entries
  sp_v.report();

  dvector  d_v(10); 
  d_v[3] = 1.6;
  
  d_v.report();
  
  // read entry
  double b = d_v(3);
  printf("b = d_v(3) = %g\n", b);


}


