/*
 * Author: WWS
 *
 * struct mat - definitions
 *
 * Purpose: simple matrix struct and associated functions
 */

#include "mat.h"

/* constructor - initialize to default state */
void mat_construct(struct mat * m ) {
  m->nrows=0;
  m->ncols=0;
  m->mdata=NULL;
}

/* initialize to hole nr x nc matrix */
int mat_init(struct mat * m, int nr, int nc) {
  if (nr<1 || nc <1) {
    fprintf(stderr,"Error: mat_init - must have at least one row, col\n");
    return E_BOUNDS;
  }
  m->nrows=nr;
  m->ncols=nc;
  if (!(m->mdata=(double *)malloc(nr*nc*sizeof(double)))) {
    fprintf(stderr,"ERROR: mat_create\n");
    fprintf(stderr,"  failed to allocated %d doubles\n",nr*nc);
    return E_ALLOC;
  }

  return 0;
}

/* destructor - deallocate, return to default state */ 
void mat_destroy(struct mat * m) {
  if (m->mdata) free(m->mdata);
  mat_construct(m);
}

/* print to stdout */
void mat_print(struct mat m) {
  int i,j;
  printf("matrix structure with %d rows and %d cols\n",m.nrows,m.ncols);
  for (i=0;i<m.nrows;i++) {
    for (j=0;j<m.ncols;j++) printf("%14.6e",m.mdata[i+j*m.nrows]);
    printf("\n");
  }
}

/* print to arbitrary file */
void mat_fprint(struct mat m, FILE * fp) {
  int i,j;
  printf("matrix structure with %d rows and %d cols\n",m.nrows,m.ncols);
  for (i=0;i<m.nrows;i++) {
    for (j=0;j<m.ncols;j++) fprintf(fp,"%14.6e",m.mdata[i+j*m.nrows]);
    fprintf(fp,"\n");
  }
}

/* initialize from file, assumed to be ascii representation 
   of matrix as rectangular array of decimal numbers. These
   can be fixed or floating point.

   designed after K&R word-counting program

   read data via sscanf into row-major storage buffer, transpose
   onto output matrix storage buffer at end - less of a sacrifice
   than it might seem, since you can't know how much storage the
   matrix needs until the end - so can't allocate its double array
   until then. Thus temporary storage and a copy operation are
   inevitable consequences of storing the matrix in human-readable
   natural form, without any redundant data.

   macro definitions in mat.h
*/
int mat_finit(struct mat * m, FILE * fp) {

  int err=0;                 // error flag
  int ir,ic,iw,j,k,nr,nc;    // counters, accumulators
  int state;                 // in, out of word - from K&R 
  char s[MAXC];              // input string buffer
  char w[MAXC];              // word buffer to build decimal numbers
  double buf[MAXD];          // row-major storage, transposed onto output

  ir=-1;                     // row count - incremented first thing
  iw=0;                      // word count
  nr=0;                      // number of rows
  nc=0;                      // number of cols

  /* grab a line until none are left (EOF) */
  while (fgets(s,MAXC,fp)) {
    // start outside word
    state=OUT;
    // reading another row...
    ir++;
    // j=char # in line
    // k=char # in word
    // ic = column counter
    j=0;k=0;ic=0;
    // work your way along the line until end (newline)
    while (s[j] != '\n') {
      // if we're in a word, read next char into word buffer
      if (state==IN) {
	w[k]=s[j];
	// increment counters
	j++;k++;
	// if next char is whitespace, change state, terminate
	// word, and scan into double buffer, using general
	// format conversion for output of doubles
	if ((s[j]==' ')||(s[j]=='\n')) {
	  state=OUT;
	  w[k]='\0';
	  sscanf(w,"%lg",&(buf[iw]));
	  // next column, next word
	  ic++;
	  iw++;
	  // don't overrun the buffer!
	  if (iw>MAXD-1) {
	    fprintf(stderr,"Error: mat_read\n");
	    fprintf(stderr,"Oops...too many entries\n");
	    return E_BOUNDS;
	  }
	  // end-of-line action - new row
	  if (s[j]=='\n') nr++;
	}
      }
      // otherwise we're outside of word
      else {
	// if the current char is non-whitespace, then we're int
	if (!(s[j]==' ') && !(s[j]=='\n')) {
	  state=IN;
	  // start char count in word
	  k=0;
	  // start word
	  w[0]=s[j];
	}
	// otherwise just bump up the char count in line, go to next
	else {
	  j++;
	}
      }
    }

    // if you get to here, you've read a newline, i.e. we are at the end
    // of a line.
    // at end of first line (ir=0), record number of cols
    if (ir==0) {
      nc=ic;
    }
    else {
      // otherwise, check that number of words is number of cols
      if (ic != nc) {
	fprintf(stderr,"Error: mat_read\n");
	fprintf(stderr,"didn't read same number of cols as prev rows\n");
	return E_BOUNDS;
      }
    }
  }

  // if you get to here, you've come to eof
  // then the number of rows is the row count (from zero) plus one
  nr=ir+1;
  
  // allocate matrix
  if (err=mat_init(m,nr,nc)) {
    fprintf(stderr,"Error: mat_read from mat_init\n");
    return err;
  } 

  // transpose row-major storage onto column-major output
  for (ir=0;ir<nr;ir++) {
    for (ic=0;ic<nc;ic++) {
      m->mdata[ir+ic*nr]=buf[ic+ir*nc];
    }
  }

  // should be returning 0 if we get this far
  return err;

}

/* vector initialization from file - basically, re-use matrix
 * initialization code. Comments should be same...
 */
int vec_finit(struct vec * v, FILE * fp) {

  int err=0;
  int ir,ic,iw,j,k,nr,nc;
  int state;
  char s[MAXC];
  char w[MAXC];
  double buf[MAXD];

  ir=-1;
  iw=0;
  nr=0;
  nc=0;

  while (fgets(s,MAXC,fp)) {
    state=OUT;
    ir++;
    j=0;k=0;ic=0;

    // a difference from matrix case - stop reading line as soon 
    // as you've read one col!
    while (s[j] != '\n' && !ic) {
      if (state==IN) {

	w[k]=s[j];
	j++;k++;
	if ((s[j]==' ')||(s[j]=='\n')) {
	  state=OUT;
	  w[k]='\0';

	  sscanf(w,"%lg",&(buf[iw]));
	  ic++;
	  iw++;
	  if (iw>MAXD-1) {
	    fprintf(stderr,"Error: mat_read\n");
	    fprintf(stderr,"Oops...too many entries\n");
	    return E_BOUNDS;
	  }
	  if (s[j]=='\n') nr++;
	}
      }
      else {
	if (!(s[j]==' ') && !(s[j]=='\n')) {
	  state=IN;
	  k=0;
	  w[0]=s[j];
	}
	else {
	  j++;
	}
      }
    }
    if (ir==0) {
      nc=ic;
    }
    else {
      if (ic != nc) {
	fprintf(stderr,"Error: mat_read\n");
	fprintf(stderr,"didn't read same number of cols as prev rows\n");
	return E_BOUNDS;
      }
    }
  }
  nr=ir+1;
  
  if (err=vec_init(v,nr)) {
    fprintf(stderr,"Error: mat_read from mat_init\n");
    return err;
  } 
  for (ir=0;ir<nr;ir++) {
    v->vdata[ir]=buf[ir];
  }
  return err;

}
