/* 
 * Author: WWS
 * 
 * Purpose: reads matrix in natural form from terminal,
 * followed by blank line, followed by vector (in column).
 * Prints out matrix-vector product. 
 *
 * This implementation is somewhat wasteful of storage.
 * There is no avoiding reading the matrix in row-major,
 * since that's how it comes from the terminal. I store
 * the result in a big buffer, then transpose it into 
 * the storage reserved for the matrix. In fact it's possible
 * at the cost of some more effort to switch between row-major
 * and column-major forms "in place", that is, using only one
 * block of storage - the idea is to swap entries symmetrically
 * placed about the diagonal. However that's more work.
 */

#include <stdio.h>
#include <stdlib.h>

/* designed to handle 20x20 matrices at most - this
   storage limit is checked in the obvious places.
*/
#define MAXMAT 400  
#define MAXVEC 20

/* can't handle lines that are too long */
#define MAXC   128

/* in/out of words states from K&R word counting example */
#define IN     1
#define OUT    0

/* multiplies matrix m times vector x, output in vector b. assumes
   that at least nrows*ncols floats have been allocated to m, and
   similarly for x, b - no error checking is possible! Assumes that
   m has been stored column-major.
*/
void matmult(int nrows, int ncols, float * m, float * x, float * b) {
  int i, j;          /* counters */

  /* do it */
  for (i=0;i<nrows;i++) {
    b[i]=0.0f;   /* initialize b */
    for (j=0;j<ncols;j++) b[i]+=m[i+j*nrows]*x[j];
  }
}

int main(int argc, char ** argv) {

  /* allocate storage for matrix and vectors */
  float m[MAXMAT];
  float x[MAXVEC];
  float b[MAXVEC];

  /* counters and accumulators for nrows, ncols */
  int ir,ic,iw,j,k,nr,nc;  
  /* in or out of word */
  int state;
  /* line, word storage */
  char s[MAXC];
  char w[MAXC];
  /* flag to indicate that we're done with read */
  int done;

  /* float buffer - read matrix into this, then transpose */
  float buf[MAXMAT];

  /* initialize counters */
  ir=0;  // row counter
  iw=0;  // word counter
  nr=0;  // number of rows
  nc=0;  // number of cols
  done=0; 

  /* keep getting lines from terminal */
  while (!done && fgets(s,MAXC,stdin)) {

    state=OUT;

    /* initialize counters for this row */
    //    ir++;
    j=0;k=0;ic=0;

    /* read through line, stripping out words as in the
       K&R word counting program. The nr variable is only 
       set (nonzero) when we get to the blank line, so it
       serves as a switch between reading the matrix and reading
       the vector.
    */
    while ((s[j] != '\n') && (nr==0)) {
      if (state==IN) {
	w[k]=s[j];
	j++;k++;
	if ((s[j]==' ')||(s[j]=='\n')) {
	  state=OUT;
	  w[k]='\0';
	  /* we've found a word representing a float, 
	     so translate it using sscanf
	  */
	  sscanf(w,"%g",&(buf[iw]));
	  ic++;
	  /* check that num cols not too big */
	  if (ic>MAXVEC-1) {
	    fprintf(stderr,"Error: matv\n");
	    fprintf(stderr,"too many cols\n");
	    exit(1);
	  }
	  iw++;
	  if (iw>MAXMAT-1) {
	    fprintf(stderr,"Error: matv\n");
	    fprintf(stderr,"Oops...too many entries\n");
	    exit(1);
	  }
	  /* a newline signifies the end of a row, so count it */
	  if (s[j]=='\n') {
	    ir++;
	    if (ir>MAXVEC-1) {
	      fprintf(stderr,"Error: matv\n");
	      fprintf(stderr,"too many rows\n");
	      exit(1);
	    }
	    /* first time through, set the number of cols */
	    if (ir==1) nc=ic;
	  }
	}
      }
      else {
	if (!(s[j]==' ') && !(s[j]=='\n')) {
	  state=IN;
	  k=0;
	  w[0]=s[j];
	}
	else {
	  j++;
	}
      }
    }

    /* if we have found no cols this time, then that's the
       blank line separating vector and matrix. Reset the counters
       for rows.
    */
    if (nr==0) {
      if (ic==0) {
	nr=ir;
	ir=0;
      }
      /* all other times, check that it's consistent */
      else {
	if (ic != nc) {
	  fprintf(stderr,"Error: matv\n");
	  fprintf(stderr,"didn't read same number of cols as prev rows\n");
	  exit(1);
	}
      }
    }
    else {

      /* we have read the matrix - now read the rest of the lines, which
	 should have one float to a row. First, reset the counters -
	 in the matrix block, ic was the total number of words read in a
	 line - here, it should not go over 1.
      */
    
      j=0;
      k=0;
      ic=0;

      state=OUT;

      while (s[j] != '\n' && ir<nc && ic<1) {
	if (state==IN) {
	  w[k]=s[j];
	  j++;k++;
	  if ((s[j]==' ')||(s[j]=='\n')) {
	    state=OUT;
	    w[k]='\0';
	    /* we've found a word representing a float, 
	       so translate it using sscanf
	    */
	    sscanf(w,"%g",&(x[ir]));
	    /* bump up number of rows read*/
	    ir++;
	    /* increment ic - this will bump us out
	       of while loop
	    */
	    ic++;
	  }
	}
	else {
	  if (!(s[j]==' ') && !(s[j]=='\n')) {
	    state=IN;
	    k=0;
	    w[0]=s[j];
	  }
	  else {
	    j++;
	  }
	}

      }
      /* the length of the input vector should be the
	 number of cols - if we have read that many
	 elements, we're done!
      */
      if (ir==nc) done=1;
    }
  }
  
  /* matrix stored in buf in row-major - transpose to get it
     into col major. This can be done "in place", ie. without
     the extra storage buffer, but it's trickier.
  */
  for (ir=0;ir<nr;ir++) {
    for (ic=0;ic<nc;ic++) {
      m[ir+ic*nr]=buf[ic+ir*nc];
    }
  }

  /* finally, call matmult */
  matmult(nr,nc,m,x,b);

  /* and print out the result */
  printf("\n");
  for (ir=0;ir<nr;ir++)
    printf("%12.4e\n",b[ir]);

}
  
