/*
 * Author: WWS
 *
 * Purpose: define simple data type for vector
 * storage, equivalent to GSL block.
 */

/* 
 * ALWAYS PUT A SIMILAR #ifdef AROUND YOUR HEADER
 * MATERIAL!!!!
 *
 * inserted to prevent multiple declarations from 
 * occuring when multiple copies of this file are
 * included. use lots of underscores to make coincidence
 * of names unlikely.
 */
#ifndef __WWS_VEC__
#define __WWS_VEC__

/* 
 * the usual suspects
 */

#include "cstd.h"

/*
 * error flags - have same meaning throughout
 * note that if more than one file were involved,
 * these should be located in a header which can 
 * be included in all files
 */

#define E_ALLOC 1
#define E_BOUNDS 2

/*
 * an array that knows its length - arguably, a vector
 */
struct vec {
  int len;
  double * vdata;
};

/*
 * sets a vector to its initial state - should be called
 * automatically when a vector is created, but C provides 
 * no mechanism to do this, so you have to do it by hand
 *
 * Note the use of the "->" syntax - this is equivalent to
 * dereferencing followed by member access - that is, we
 * could as well have written
 *
 * (*v).len=0;
 * (*v).vdata=NULL;
 *
 * The parens are necessary to force the compiler to dereference THEN
 * find member - the structure member operator "." has higher
 * precedence than almost anything else, so "*v.len" first tries to
 * find the member len of the struct v - but v is not a struct, but a
 * pointer to one.
 *
 */
void vec_construct(struct vec * v);

/*
 * I wrote this initialization function so that the "ideal" initial
 * status is verified - making memory leaks more unlikely. Otherwise,
 * you could call this function repeatedly without freeing the data
 * member. Each time, you would lose control of the previously 
 * allocated block of memory (which data pointed to, before being
 * overwritten!).
 */
int vec_init(struct vec * v, int n);

/* after destruction, you can re-create another vector in the same
 * variable. Since the v memory has been freed, it will likely be
 * re-used (but not necessarily - some other part of the process might
 * use it.)
 */
void vec_destroy(struct vec * v);

/*
 * you can access the components of v by v.vdata[i] - the point
 * of this function is the bounds check. That is,
 *
 * x=vec_get(v,i);
 * 
 * gives the same result as 
 *
 * x=v.vdata[i];
 *
 * if i is in range, but gives you an error message instead of a 
 * seg fault otherwise.
 */
double vec_get(struct vec v, int i);

/* 
 * same story - why is it sufficient to pass a vec rather than 
 * a vec *?
 */
void vec_set(struct vec v, int i, double x);

/* a simple display function */
void vec_print(struct vec v);

/* a function to print the vector to a file */
void vec_fprint(struct vec v, FILE * fp);

/*
 * DON'T FORGET THE END OF YOUR #ifndef...#define... BLOCK!!
 */
#endif
