#include "vecpp.hh"

/*
 * Since member functions belong to a class, the class name is really
 * part of their signature. The syntax to indicate this membership is
 * [class name]::[function name]. Definitions outside of the
 * declaration (header) file must be identified in this way so that
 * they can be associated with the correct functions via the
 * declarations.
 *
 * Since the class is identified, names refer to class members unless
 * otherwise indicated (will see how that's done by and by). This is
 * true both for data members like len and vdata, and for member
 * functions (methods). Thus "len" in the following functions refers
 * to the "len" member of an instance of VecPP, not to some other
 * variable of the same name that may be floating around outside the
 * VecPP class code.
 */

VecPP::VecPP() {
  len=0;
  vdata=NULL;
}

VecPP::~VecPP() {
  if (vdata) delete [] vdata;
}

// default value declarations belong ONLY in the header file
void VecPP::init(int n, double c) {
  if (vdata) {
    cerr<<"Error: VecPP::init\n";
    cerr<<"  cannot initialize - already initialized!\n";
    exit(1);
  }
  if (n < 1) {
    cerr<<"Error: VecPP::init\n";
    cerr<<"  requested length = "<<n<<" not positive\n";
    exit(1);
  }
  len = n;
  vdata = new double[n];
  /*
   * chose to initialize the vector to a const function of index,
   * default value = 0.0. Note the ability to place the declaration of
   * the counter inside the for scope, thus avoiding cluttering up the
   * code with counter declarations.
   */
  for (int i=0;i<len;i++) vdata[i]=c;
}

int VecPP::size() const { return len; }

void VecPP::get(int i, double & c) const {
  if (i<0 || i>len-1) {
    cerr<<"Error: VecPP::get\n";
    cerr<<"requested index = "<<i<<" outside of legal range [0, "
	<<len-1<<"]\n";
    exit(1);
  }
  c=vdata[i];
}

void VecPP::set(int i, double c) {
  if (i<0 || i>len-1) {
    cerr<<"Error: VecPP::set\n";
    cerr<<"requested index = "<<i<<" outside of legal range [0, "
	<<len-1<<"]\n";
    exit(1);
  }
  vdata[i]=c;
}

ostream & VecPP::write(ostream & str) const {
  str<<"instance of VecPP array class\n";
  str<<"data members:\n";
  str<<"len = "<<len<<"\n";
  for (int i=0;i<len;i++) 
    str<<"vdata["<<i<<"] = "<<vdata[i]<<"\n";
  return str;
}
  
