/*
 * Author: WWS
 * 
 * Purpose: a straightforward morph of the struct vec (example 42 and
 * related files) into a simple class. Shows how access control,
 * constructors, destructors, const and non-const member functions
 * work, and suggest how these features can help the programmer avoid
 * common problems.
 */

#ifndef __CAAM420_VECPP__
#define __CAAM420_VECPP__

// the usual
#include "cppstd.hh"

// syntax for simple class declaration
class VecPP {

  /*
   * the private keyword precedes declarations of variables (and
   * functions, also called (class) methods) that are not accessible
   * (directly) by any functions other than class methods of the same
   * class. This allows the programmer to strictly regulate the
   * precise manner in which these variables may be assigned. In this
   * case, external code (main, some other function not a member of
   * the VecPP class) is not permitted to assign a value pointed to by
   * vdata, or to free the memory associated with it, or change the
   * len parameter to be inconsitent with vdata, or perform any other
   * type of mischief.
   *
   * Many programmer place their private variable declarations at the
   * bottom of the declaration scope. I prefer to see them up-front,
   * as they are often the heart of the data type.
   */ 

private:

  int len;
  double * vdata;

public:

  /* default constructor - takes no arguments, looks like a
   * declaration, but does what vec_construct did - however MUST BE
   * CALLED to create a VecPP instance, so the desired initial state
   * of the internal variables is ALWAYS attained.
   */
  VecPP();

  /* destructor - always called when a VecPP object declared by call
   * to the default constructor goes out of scope. Must be called "by
   * hand" on instances created by operator new.
   */
  ~VecPP();

  /* post-construction initialization - arranged so that it can only
   * be called once, as in vec, to avoid memory leaks. If you try to
   * call it again, something bad should happen. This version does the
   * simplest thing - kills the entire process. Could have returned an
   * int, or a bool, but that would not FORCE the calling function to
   * acknowledge the error condition. More satisfactory ways of
   * handling errors are to come.
   */
  void init(int n, double c=0.0);

  /* since the array length is now a private attribute, you must give
   * external functions explicit access in order to use it.  If you
   * return it as a value (which is a copy of a variable value in the
   * size() function), then the internal state of the VecPP instance
   * cannot possibly change, so you can declare this function as
   * const.
   */
  int size() const;

  /* extract a value - an indexing read operator with range
   * checking. this function does not change the internal state of the
   * object, so it is declared const. Elected to write it with a
   * passed reference, rather than returning a double.
   */
  void get(int i, double & c) const;

  /* this function DOES change the internal state of the object, so
   * SHOULD NOT be declared const.
  */
  void set(int i, double c);

  /* analogous to vec_fprint - but writes data to a C++ stream, 
   * rather than a C stream.
   */
  ostream & write(ostream & str) const;
};

// NOTE: the whole thing between {...} is  a declaration, so MUST end
// with a semicolon - leaving this off is an easy mistake to make.

#endif
