#ifndef __CAAM420_VECPPBASE__
#define __CAAM420_VECPPBASE__

#include "cppstd.hh"

/*
 * Author: WWS
 *
 * Purpose: first cut at base class for VecPP hierarchy. Adapted
 * from Warburton's "skinny base class".
 */

/* 
 * Common programming practice: 
 * 1. class names begin with upper case, and are nouns:
 *    MyVector, DenseMatrix,...
 * 2. function names begin with lower case and are verbs:
 *    get(), setValue(), initializeInternalStruct(),...
 * 3. avoid internal underscores if possible - separate name
 *    components with capitalization. Internal spaces are illegal,
 *    of course.
 */
class VecPPBase {

  /* if we insert a "dim" int as private or protected data,
   * then that implementation of dimension must be inherited
   * by all subclasses. Will see that this is not always convenient.
   * however, so instead make dimension-revealing function pure
   * virtual so that all instantiable subclasses must implement it
   */
 
  
public:

  VecPPBase() { 
    //    cout<<"VecPPBase constructor\n"; 
  }

  /* there are no data members, so the default constructor and destructor
   * provided by the compiler are just fine. However it is NECESSARY to 
   * declare the destructor to be virtual, so that it can be overridden by 
   * subclasses, and thus whatever actual cleanup is needed actually takes
   * place when a VecPPBase object is destroyed (see Stroustrup p. 319).
   */
  virtual ~VecPPBase() { 
    //    cout<<"VecPPBase destructor\n";
  }

  /* public length function - must be implemented in instantiable 
     subclasses. Does not change any conceivable internal data, so 
     should be declared const. 
  */
  virtual int getLength() const = 0;

  /* a solution to the element assignment problem, more "C++-ish": use
     call-by-reference, and provide both const and non-const
     references. The latter can be assigned!
  */
  virtual double & operator()(int i) = 0;
  virtual double const & operator()(int i) const = 0;

  /* basic linear algebra op, 
   *
   * y <- a*x + y
   *
   * implemented but virtual so can be overridden in derived class 
   *
   * very simple example of an abstract algorithm - uses only functions
   * defined for all VecPPBase instances. no matter what concrete subclass
   * they come from.
   */
  virtual void axpy(double a, VecPPBase const & x) {
    if (x.getLength() != this->getLength()) {
      cerr<<"Error: VecPPBase::axpy\n";
      cerr<<"incompatible vectors:\n";
      cerr<<"length of this = "<<this->getLength();
      cerr<<"length of that = "<<x.getLength();
      exit(1);
    }
    for (int i=0;i<getLength();i++) (*this)(i)+=a*x(i);
  }

  /*
   * standard Euclidean dot product, can be overridden 
   */
  virtual double dot(VecPPBase const & x) const {
    if (x.getLength() != this->getLength()) {
      cerr<<"Error: VecPPBase::dot\n";
      cerr<<"incompatible vectors:\n";
      cerr<<"length of this = "<<this->getLength();
      cerr<<"length of that = "<<x.getLength();
      exit(1);
    }
    double d=0.0;
    for (int i=0;i<getLength();i++) d += ((*this)(i))*(x(i));
    return d;
  }

  /* 
   * assign zero vector - the only coordinate-invariant assignment.
   * of course this is not a const function.
   */
  virtual void zero() {
    for (int i=0;i<this->getLength();i++) (*this)(i)=0.0;
  }

  /* mandatory report function */
  virtual ostream & report(ostream & str) const = 0;

};

#endif


